@@ -79,11 +79,24 @@ async def _worker(executor_weakref, work_queue):
             del executor
 
 
-def _async_loop_thread(loop):
+async def _loop_send_stop_event(stop_event):
+    """Set stop event to stop event loop.
+    """
+    stop_event.set()
+
+
+async def _loop_main(stop_event):
+    """Event loop main function
+    that blocks until stop event is set.
+    """
+    await stop_event.wait()
+
+
+def _async_loop_thread(loop, stop_event):
     """Async loop thread.
     """
     asyncio.set_event_loop(loop)
-    loop.run_forever()
+    loop.run_until_complete(_loop_main(stop_event))
 
 
 class AsyncPoolExecutor(_base._base.Executor):
@@ -98,6 +111,7 @@ class AsyncPoolExecutor(_base._base.Executor):
         self._open = False
         self._max_workers = max_workers
         self._loop = loop or asyncio.new_event_loop()
+        self._loop_stop_event = asyncio.Event(loop=self._loop)
         self._work_queue = asyncio.Queue(loop=self._loop)
         self._tasks = set()
         self._shutdown = False
@@ -111,12 +125,14 @@ class AsyncPoolExecutor(_base._base.Executor):
         return bool(self._open)
 
     def __enter__(self):
-        if not self._open:
-            self._async_loop_thread = threading.Thread(target=_async_loop_thread,
-                kwargs={"loop": self._loop}, daemon=True)
-            self._async_loop_thread.start()
-            self._open = True
-        return self
+        with self._shutdown_lock:
+            if not self._open:
+                self._async_loop_thread = threading.Thread(target=_async_loop_thread,
+                    kwargs={"loop": self._loop, "stop_event": self._loop_stop_event},
+                    daemon=True)
+                self._async_loop_thread.start()
+                self._open = True
+            return self
 
     def submit(self, fn, args=None, kwargs=None, block=True):
         if args is None:
@@ -140,9 +156,13 @@ class AsyncPoolExecutor(_base._base.Executor):
             idle_workers = self._adjust_task_count()
 
             if (idle_workers or block) and self._max_workers > 0:
+                if is_running_in_event_loop() and asyncio.get_event_loop() is self._loop:
+                    raise RuntimeError("deadlock detected")
                 asyncio.run_coroutine_threadsafe(self._work_queue.put(work_item), loop=self._loop).result()
 
         if (not block and not idle_workers) or self._max_workers < 1:
+            if is_running_in_event_loop() and asyncio.get_event_loop() is self._loop:
+                raise RuntimeError("deadlock detected")
             asyncio.run_coroutine_threadsafe(work_item.run(), loop=self._loop).result()
 
         if is_running_in_event_loop():
@@ -196,12 +216,11 @@ class AsyncPoolExecutor(_base._base.Executor):
                 if exc is not None:
                     raise exc
             finally:
-                # we can't close the pool as it might be needed
-                # for cleanups so live it running
-                # The following resources are left behind:
-                #   self._loop - event loop
-                #   self._async_loop_thread - daemon thread that is left running
-                pass
+                asyncio.run_coroutine_threadsafe(
+                        _loop_send_stop_event(self._loop_stop_event), loop=self._loop
+                    ).result()
+                if self._async_loop_thread is not None:
+                    self._async_loop_thread.join()
 
 
 class GlobalAsyncPoolExecutor(AsyncPoolExecutor):
@@ -212,3 +231,6 @@ class GlobalAsyncPoolExecutor(AsyncPoolExecutor):
             raise ValueError("max_workers must be positive or 0")
         super(GlobalAsyncPoolExecutor, self).__init__(
             max_workers=max_workers-1, task_name_prefix=task_name_prefix, _check_max_workers=False)
+
+    def submit(self, fn, args=None, kwargs=None, block=False):
+        return super(GlobalAsyncPoolExecutor, self).submit(fn=fn, args=args, kwargs=kwargs, block=block)
