@@ -38,5 +38,7 @@ database = None
 show_skipped = False
 #: randomize order of loaded tests
 random_order = False
-#: global parallel pool
-global_parallel_pool = None
\ No newline at end of file
+#: global thread pool
+global_thread_pool = None
+#: global async pool
+global_async_pool = None
